@@ -9,24 +9,26 @@ use vars qw(@EXPORT @EXPORT_OK @ISA $VERSION);
 BEGIN {
     require Exporter;
     @ISA = qw(Exporter);
-    $VERSION = '1.00';
+    $VERSION = '1.01';
     @EXPORT = @EXPORT_OK = qw(rd_f wr_f ap_f find_temp find_bin lockafile unlockafile unlock_all_the_files);
 }
 
-our $OK    = 'OK';  # one of "OK", "WARNING" or "ERROR", reflecting most recently performed operation
-our $ERROR = '';    # short invariant description of error, or empty string if none
-our $ERRNO = '';    # variant description of error (such as $!), or empty string if none
+our $OK     = 'OK'; # one of "OK", "WARNING" or "ERROR", reflecting most recently performed operation
+our $ERROR  = '';   # short invariant description of error, or empty string if none
+our $ERRNO  = '';   # variant description of error (such as $!), or empty string if none
+our $ERRNUM =  0;   # numerical variant description of error (such as $!), or empty string if none, undocumented, only used for unit tests
 our %LOCKS_HASH;    # keys on lockfile to bind count, for supporting nested locks.
 
 sub rd_f {
     my ($fn) = @_;
     my ($fh, $buf);
     if (!defined($fn) || ($fn eq '')) {
-        ($OK, $ERROR, $ERRNO) = ('ERROR', 'no filename supplied', -1);
+        ($OK, $ERROR, $ERRNO, $ERRNUM) = ('ERROR', 'no filename supplied', -1, 0);
         return undef;
     }
+    $! = 0;
     unless (open($fh, '<:raw', $fn)) {
-        ($OK, $ERROR, $ERRNO) = ('ERROR', "cannot open for reading", $!);
+        ($OK, $ERROR, $ERRNO, $ERRNUM) = ('ERROR', "cannot open for reading", $!, 0+$!);
         return undef;
     }
     my $file_size = -s $fn;
@@ -34,23 +36,23 @@ sub rd_f {
         my $n_bytes = read($fh, $buf, $file_size);
         close($fh);
         if (!defined($n_bytes)) {
-            ($OK, $ERROR, $ERRNO) = ('ERROR', 'read failed', $!);
+            ($OK, $ERROR, $ERRNO, $ERRNUM) = ('ERROR', 'read failed', $!, 0+$!);
             return undef;
         }
         elsif ($n_bytes != $file_size) {
-            ($OK, $ERROR, $ERRNO) = ('ERROR', 'read underflow', $!);
+            ($OK, $ERROR, $ERRNO, $ERRNUM) = ('ERROR', 'read underflow', $!, 0+$!);
             return undef;
         }
     }
     else {
         local $/;
         if (!defined($buf = readline($fh))) {
-            ($OK, $ERROR, $ERRNO) = ('ERROR', 'read failed', $!);
+            ($OK, $ERROR, $ERRNO, $ERRNUM) = ('ERROR', 'read failed', $!, 0+$!);
             return undef;
         }
     }
     close($fh);
-    ($OK, $ERROR, $ERRNO) = ('OK', '', '');
+    ($OK, $ERROR, $ERRNO, $ERRNUM) = ('OK', '', '', 0);
     return $buf;
 }
 
@@ -58,19 +60,20 @@ sub wr_f {
     my ($fn, $buf) = @_;
     my $fh;
     if (!defined($fn) || ($fn eq '')) {
-        ($OK, $ERROR, $ERRNO) = ('ERROR', 'no filename supplied', -1);
+        ($OK, $ERROR, $ERRNO, $ERRNUM) = ('ERROR', 'no filename supplied', -1, 0);
         return undef;
     }
+    $! = 0;
     unless (open($fh, '>:raw', $fn)) {
-        ($OK, $ERROR, $ERRNO) = ('ERROR', "cannot open for writing", $!);
+        ($OK, $ERROR, $ERRNO, $ERRNUM) = ('ERROR', "cannot open for writing", $!, 0+$!);
         return undef;
     }
     unless (print $fh $buf) {
-        ($OK, $ERROR, $ERRNO) = ('ERROR', 'write error', $!);
+        ($OK, $ERROR, $ERRNO, $ERRNUM) = ('ERROR', 'write error', $!, 0+$!);
         return undef;
     }
     close($fh);
-    ($OK, $ERROR, $ERRNO) = ('OK', '', '');
+    ($OK, $ERROR, $ERRNO, $ERRNUM) = ('OK', '', '', 0);
     return 'OK';
 }
 
@@ -78,19 +81,20 @@ sub ap_f {
     my ($fn, $buf) = @_;
     my $fh;
     if (!defined($fn) || ($fn eq '')) {
-        ($OK, $ERROR, $ERRNO) = ('ERROR', 'no filename supplied', -1);
+        ($OK, $ERROR, $ERRNO, $ERRNUM) = ('ERROR', 'no filename supplied', -1, 0);
         return undef;
     }
+    $! = 0;
     unless (open($fh, '>>:raw', $fn)) {
-        ($OK, $ERROR, $ERRNO) = ('ERROR', "cannot open for appending", $!);
+        ($OK, $ERROR, $ERRNO, $ERRNUM) = ('ERROR', "cannot open for appending", $!, 0+$!);
         return undef;
     }
     unless (print $fh $buf) {
-        ($OK, $ERROR, $ERRNO) = ('ERROR', 'append error', $!);
+        ($OK, $ERROR, $ERRNO, $ERRNUM) = ('ERROR', 'append error', $!, 0+$!);
         return undef;
     }
     close($fh);
-    ($OK, $ERROR, $ERRNO) = ('OK', '', '');
+    ($OK, $ERROR, $ERRNO, $ERRNUM) = ('OK', '', '', 0);
     return 'OK';
 }
 
@@ -118,10 +122,10 @@ sub find_temp {
     push(@_, '/dev/shm') unless ($is_windows); # Lowest priority, since this is typically a ramdisk.
     foreach my $d (@_) {
         next unless (-d $d);
-        ($OK, $ERROR, $ERRNO) = ('OK', '', '');
+        ($OK, $ERROR, $ERRNO, $ERRNUM) = ('OK', '', '', 0);
         return $d if (-w _);
     }
-    ($OK, $ERROR, $ERRNO) = ('WARNING', 'no appropriate temporary directory found', '');
+    ($OK, $ERROR, $ERRNO, $ERRNUM) = ('WARNING', 'no appropriate temporary directory found', '', 0);
     return undef;
 }
 
@@ -137,10 +141,10 @@ sub find_bin {
         $been_there{$d} = 1;
         my $f = "$d/$bin_name";
         next unless (-x $f);
-        ($OK, $ERROR, $ERRNO) = ('OK', '', '');
+        ($OK, $ERROR, $ERRNO, $ERRNUM) = ('OK', '', '', 0);
         return $f;
     }
-    ($OK, $ERROR, $ERRNO) = ('WARNING', 'no executable found', '');
+    ($OK, $ERROR, $ERRNO, $ERRNUM) = ('WARNING', 'no executable found', '', 0);
     return undef;
 }
 
@@ -154,10 +158,10 @@ sub find_bin_win32 {
         $been_there{$d} = 1;
         my $f = "$d\\$bin_name";
         next unless (-x $f);
-        ($OK, $ERROR, $ERRNO) = ('OK', '', '');
+        ($OK, $ERROR, $ERRNO, $ERRNUM) = ('OK', '', '', 0);
         return $f;
     }
-    ($OK, $ERROR, $ERRNO) = ('WARNING', 'no executable found', '');
+    ($OK, $ERROR, $ERRNO, $ERRNUM) = ('WARNING', 'no executable found', '', 0);
     return undef;
 }
 
@@ -186,7 +190,7 @@ sub lockafile {
 
             if (!defined($txt)) {  # handling potential race condition or naughty unreadable lockfile
                 if ((time() - $tm_start) > $opt{limit}) {
-                    ($OK, $ERROR, $ERRNO) = ('WARNING', 'lockfile racy or unreadable', $lockfile_name);
+                    ($OK, $ERROR, $ERRNO, $ERRNUM) = ('WARNING', 'lockfile racy or unreadable', $lockfile_name, 0);
                     return 0;
                 }
                 select(undef, undef, undef, $opt{sleep_duration});
@@ -205,7 +209,7 @@ sub lockafile {
         }
 
         if ((time() - $tm_start) > $opt{limit}) {
-            ($OK, $ERROR, $ERRNO) = ('OK', '', '');  # Not an error; simply unable to acquire lock within specified duration
+            ($OK, $ERROR, $ERRNO, $ERRNUM) = ('OK', '', '', 0);  # Not an error; simply unable to acquire lock within specified duration
             return 0;
         }
 
@@ -215,7 +219,7 @@ sub lockafile {
     syswrite($lockfile_fh, $msg);
     close($lockfile_fh);
     $LOCKS_HASH{$f} = 1;
-    ($OK, $ERROR, $ERRNO) = ('OK', '', '');
+    ($OK, $ERROR, $ERRNO, $ERRNUM) = ('OK', '', '', 0);
     return 1;
 }
 
@@ -224,22 +228,23 @@ sub unlockafile {
     my $lockfile_fh;
     my $dgram;
     my $lockfile_name = $opt{lockfile_name} || "$f.lock";
-    ($OK, $ERROR, $ERRNO) = ('OK', '', '');
+    ($OK, $ERROR, $ERRNO, $ERRNUM) = ('OK', '', '', 0);
 
     # TODO - This is fast and simple, but fails to handle expired lockfiles
     if ($LOCKS_HASH{$f}) {
         $LOCKS_HASH{$f}--;
         return 1 if ($LOCKS_HASH{$f} > 0);
     }
+    $! = 0;
     unless (sysopen($lockfile_fh, $lockfile_name, &O_RDONLY)) {
-        ($OK, $ERROR, $ERRNO) = ('ERROR', 'open failure', $!);
+        ($OK, $ERROR, $ERRNO, $ERRNUM) = ('ERROR', 'open failure', $!, 0+$!);
         return 0;
     }
     unless(my $result = sysread($lockfile_fh, $dgram, 4095)) {
         if (defined($result)) {
-            ($OK, $ERROR, $ERRNO) = ('ERROR', 'read zero bytes from lockfile', '');
+            ($OK, $ERROR, $ERRNO, $ERRNUM) = ('ERROR', 'read zero bytes from lockfile', '', 0+$!);
         } else {
-            ($OK, $ERROR, $ERRNO) = ('ERROR', 'read error', $!);
+            ($OK, $ERROR, $ERRNO, $ERRNUM) = ('ERROR', 'read error', $!, 0+$!);
         }
         return 0;
     }
@@ -249,7 +254,7 @@ sub unlockafile {
     $LOCKS_HASH{$f} = 0;
     if (defined($lpid) && ($lpid ne $$)) {
         # oops! not ours anymore
-        ($OK, $ERROR, $ERRNO) = ('ERROR', 'lost lock', $dgram);
+        ($OK, $ERROR, $ERRNO, $ERRNUM) = ('ERROR', 'lost lock', $dgram, 0+$!);
         return 0;
     }
     unlink($lockfile_name);
